package main;

import java.awt.image.BufferedImage;

import color.Detector;
import robot.Robot;
import servers.Server;


public class Main {

	
	public static String path="/home/pi/opencv/build/lib/";	// path of OpenCV .so libraries
	public static boolean raspberry=true;					// program runs on a raspberry pi (automatic detection)

	// camera interface
	private Camera camera;
	private Detector detector;
	
	// robot controller
	private Robot robot;
	private int mode=0;	// 0 manual, 1 automatic
	
	// servers
	private Server server;
	
	
	// Main system
	private boolean run=true;
	private int  counter=0;
	public BufferedImage output;
	
	/////////////////////////////////////////////////////////////////////////////////////////
	
	public Main(){
		

		// camera grabber
		camera=new Camera();
		detector=new Detector();
		
		// robot controller
		robot=new Robot();
		
		// servers
		server=new Server(this);
		
		
		try {Thread.sleep(20);
		} catch (InterruptedException e) {e.printStackTrace();}	
		
		/////////////////
		// used to measure execution time
		long time1=System.nanoTime()/1000000;
		long delay1=0;
		short nb=0;
		/////////////////
		
		
		// main execution loop
		while (run){
			
			/////////////////
			// measure time between frames
			delay1+=System.nanoTime()/1000000-time1;
			time1=System.nanoTime()/1000000;
			
			nb++;
			if (nb>20){
				nb=0;
				server.broadcast("framerate "+(delay1/20));
				delay1=0;
			}
			/////////////////
			
			// get new frame
			camera.read();
			
			if (counter==0) {	// process only one image on 2
				// detect color
				detector.detect(camera.img);
			
				// image sent to web interface
				output=detector.Matrix2bufferedImage();
			
				// automatic control of the robot to rotate toward a colored object
				if (mode==1 && detector.value>500) robot.action(detector.px);
				
				counter=1;
			}
			else counter--;
			
		}
		
		// close everything
		server.stop();
		robot.stop();
		
		System.out.println("########## system stopped ##########");
		System.exit(0);
	}
	
	/////////////////////////////////////////////////////////////////////////////////////////
	
	// interpret commands from client
	public void clientCommand(String command) {
		String[] elements=command.split(" ");
		
		// robot control commands
		if (elements.length>=2 && elements[0].equals("robot")){
			int speed=80;
			if 		(elements[1].equals("forward")) 	robot.setMotors(speed, speed);
			else if (elements[1].equals("backward")) 	robot.setMotors(-speed, -speed);
			else if (elements[1].equals("turnleft")) 	robot.setMotors(-speed, speed);
			else if (elements[1].equals("turnright")) 	robot.setMotors(speed,-speed);
			else if (elements[1].equals("stop")) 		robot.stop();
		}
		
		// system control commands
		if (elements.length>=2 && elements[0].equals("system")){
			if (elements[1].equals("stop")) run=false;		// stop main loop
			else {
				if (elements[1].equals("red"  )) detector.color=0;	// change selected color
				if (elements[1].equals("green")) detector.color=1;
				if (elements[1].equals("blue" )) detector.color=2;
			
				broadcast("color "+elements[1]);					// send color to be displayed
			}
		}
		
		// control mode change command
		if (elements.length>=1 && elements[0].equals("changemode")){
			if (mode==0) {
				mode=1;
				broadcast("mode automatic");
			}
			else {
				mode=0;
				broadcast("mode manual");
			}
		}
				
	}
	

	// send a message to the web interface
	public void broadcast(String msg) {
		server.broadcast(msg);
	}
	
	
	/////////////////////////////////////////////////////////////////////////////////////////
	public static void main(String[] args) {
		
		// detect if the program runs on raspberry pi
		if (!System.getProperty("user.name").equals("pi") && !System.getProperty("user.name").equals("root")) {
			System.out.println("User name: "+System.getProperty("user.name"));	// display user's name
			Main.raspberry=false;												// not a raspberry pi
		}
		
		new Main();	// start main algorithm
	}

}
